/*
 * Copyright (C) 2012-2013 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Helpers around the DCUI interface. */

#include "UI.h"

#include "Core.h"

namespace dcapi {

DCUIPtr UI::ui;
string UI::guid;
unordered_map<string, pair<UI::Command, string>> UI::commands;

bool UI::init(string pluginGuid) {
	if(!Core::handle()) { return false; }
	init(reinterpret_cast<DCUIPtr>(Core::handle()->query_interface(DCINTF_DCPP_UI, DCINTF_DCPP_UI_VER)), move(pluginGuid));
	return ui;
}
void UI::init(DCUIPtr coreUI, string pluginGuid) { ui = coreUI; guid = move(pluginGuid); }
DCUIPtr UI::handle() { return ui; }

void UI::addCommand(string name, Command command, string icon) {
	const auto& iter = commands.insert(std::make_pair(move(name), std::make_pair(command, move(icon)))).first;
	ui->add_command(guid.c_str(), iter->first.c_str(), commandCallback, iter->second.second.c_str());
}

void UI::removeCommand(const string& name) {
	ui->remove_command(guid.c_str(), name.c_str());
	commands.erase(name);
}

void DCAPI UI::commandCallback(const char* name) {
	commands[name].first();
}

} // namespace dcapi
