/*
 * Copyright (C) 2012-2013 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Plugin.h"

/* Include plugin SDK helpers. There are more interfaces available that can be included in the same
fashion (check the pluginsdk directory). */
#include <pluginsdk/Config.h>
#include <pluginsdk/Core.h>
#include <pluginsdk/Hooks.h>
#include <pluginsdk/Logger.h>
#include <pluginsdk/Tagger.h>
#include <pluginsdk/UI.h>
#include <pluginsdk/Util.h>

/* Plugin SDK helpers are in the "dcapi" namespace; ease their calling. */
using dcapi::Config;
using dcapi::Core;
using dcapi::Hooks;
using dcapi::Logger;
using dcapi::Tagger;
using dcapi::UI;
using dcapi::Util;

Plugin::Plugin() {
}

Plugin::~Plugin() {
	Hooks::clear();
}

Bool DCAPI Plugin::main(PluginState state, DCCorePtr core, dcptr_t) {
	static Plugin* instance;

	switch(state) {
	case ON_INSTALL:
	case ON_LOAD:
		{
			instance = new Plugin();
			return instance->onLoad(core, state == ON_INSTALL) ? True : False;
		}

	case ON_UNINSTALL:
	case ON_UNLOAD:
		{
			delete instance;
			instance = 0;
			return True;
		}

	default:
		{
			return False;
		}
	}
}

bool Plugin::onLoad(DCCorePtr core, bool install) {
	/* Initialization phase. Initiate additional interfaces that you may have included from the
	plugin SDK. */
	Core::init(core);
	if(!Config::init(PLUGIN_GUID) || !Hooks::init() || !Logger::init() || !Tagger::init() || !UI::init(PLUGIN_GUID) || !Util::init()) {
		return false;
	}

	if(install) {
		// This only executes when the plugin has been installed for the first time.
		Logger::log("The test plugin has been installed.");
	}

	// Start the plugin logic here; add hooks with functions from the Hooks interface.
	Logger::log("Test plugin loaded, watch out!");

	Hooks::Timer::onSecond([this](uint64_t tick, bool&) { return onSecond(tick); });
	Hooks::UI::onChatTags([this](UserDataPtr, TagDataPtr tags, bool&) { return onUiChatTags(tags); });

	return true;
}

bool Plugin::onSecond(uint64_t tick) {
	static uint64_t prevTick = 0;
	if(tick - prevTick >= 10*1000) {
		prevTick = tick;
		UI::handle()->play_sound("Media\\tada.wav");
	}
	return false;
}

bool Plugin::onUiChatTags(TagDataPtr tags) {
	// look for the pattern and make it bold.
	const string pattern = "ABC DEF";

	string text(Tagger::handle()->get_text(tags));
	size_t start, end = 0;
	while((start = text.find(pattern, end)) != string::npos) {
		end = start + pattern.size();
		Tagger::handle()->add_tag(tags, start, end, "b", "");
	}

	// replace instances of replPattern by replText.
	const string replPattern = "BC";
	const string replText = "haha";

	const auto delta = static_cast<int>(replText.size()) - static_cast<int>(replPattern.size());
	int offset = 0;

	end = 0;
	while((start = text.find(replPattern, end)) != string::npos) {
		end = start + replPattern.size();
		Tagger::handle()->replace_text(tags, start + offset, end + offset, replText.c_str());
		offset += delta;
	}
	return true;
}
